#!/bin/sh

SH_NAME=`pwd`
XML_NAME="VM_20240606-2.6.1.173.xml"
QCOW2_NAME="QuWAN.qcow2"
QCOW2_LOC="$(find `pwd` -name ${QCOW2_NAME}|head -1)"

if [ $(id -u) -ne 0 ]; then
    echo Not root user. Please run as root.
    exit
fi

#To generate QuWAN XML file
gen_xml() {
    cat << EOF > ${SH_NAME}/${XML_NAME}
<domain type='kvm' id='1'>
  <name>QuWAN_20240606-2.6.1.173</name>
  <memory unit='KiB'>1048576</memory>
  <currentMemory unit='KiB'>1048576</currentMemory>
  <vcpu placement='static'>1</vcpu>
  <os>
    <type arch='x86_64' machine='pc'>hvm</type>
    <boot dev='hd'/>
    <bootmenu enable='no'/>
  </os>
  <features>
    <acpi/>
    <apic/>
    <pae/>
  </features>
  <cpu mode='host-passthrough'>
    <topology sockets='1' cores='1' threads='1'/>
  </cpu>
  <clock offset='localtime'/>
  <on_poweroff>destroy</on_poweroff>
  <on_reboot>restart</on_reboot>
  <on_crash>restart</on_crash>
  <pm>
    <suspend-to-mem enabled='no'/>
    <suspend-to-disk enabled='no'/>
  </pm>
  <devices>
    <emulator>/usr/bin/qemu-system-x86_64</emulator>
    <disk type='file' device='disk'>
      <driver name='qemu' type='qcow2' cache='none'/>
      <source file='REPLACE_QCOW2_PATH' startupPolicy='optional'/>
      <target dev='vda' bus='virtio'/>
    </disk>
    <controller type='ide' index='0'>
      <alias name='ide'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x01' function='0x1'/>
    </controller>
    <controller type='usb' index='0' model='nec-xhci'>
      <alias name='usb'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x0b' function='0x0'/>
    </controller>
    <controller type='pci' index='0' model='pci-root'>
      <alias name='pci.0'/>
    </controller>
    <controller type='virtio-serial' index='0'>
      <alias name='virtio-serial0'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x0c' function='0x0'/>
    </controller>
    <interface type='bridge'>
      <source bridge='Quwan_1'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x03' function='0x0'/>
    </interface>
    <interface type='bridge'>
      <source bridge='Quwan_2'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x04' function='0x0'/>
    </interface>
    <interface type='bridge'>
      <source bridge='Quwan_3'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x05' function='0x0'/>
    </interface>
    <interface type='bridge'>
      <source bridge='Quwan_4'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x06' function='0x0'/>
    </interface>
    <interface type='bridge'>
      <source bridge='Quwan_5'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x07' function='0x0'/>
    </interface>
    <interface type='bridge'>
      <source bridge='Quwan_6'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x08' function='0x0'/>
    </interface>
    <interface type='bridge'>
      <source bridge='Quwan_7'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x09' function='0x0'/>
    </interface>
    <interface type='bridge'>
      <source bridge='Quwan_8'/>
      <model type='virtio'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x0a' function='0x0'/>
    </interface>
    <input type='tablet' bus='usb'>
      <alias name='input0'/>
    </input>
    <input type='mouse' bus='ps2'/>
    <input type='keyboard' bus='ps2'/>
    <graphics type='vnc' port='5900' autoport='yes' listen='0.0.0.0' keymap='en-us'>
      <listen type='address' address='0.0.0.0'/>
    </graphics>
    <video>
      <model type='vga' vram='16384' heads='1'/>
      <alias name='video0'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x02' function='0x0'/>
    </video>
    <memballoon model='virtio'>
      <alias name='balloon0'/>
      <address type='pci' domain='0x0000' bus='0x00' slot='0x0e' function='0x0'/>
    </memballoon>
    <serial type='pty'>
      <target port='1'/>
    </serial>
    <console type='pty'>
      <target type='serial' port='1'/>
    </console>
  </devices>
</domain>
EOF
}

modify_qcow2_path() {
    sed -i "s%REPLACE_QCOW2_PATH%${QCOW2_LOC}%g" ${XML_NAME}
}

setup_bridges_with_nmcli_command() {
    for i in `seq 8`
    do
	if [ ! -d /sys/class/net/Quwan_${i} ] ; then
	    nmcli connection add type bridge ifname Quwan_${i} stp no
	else
	    echo Quwan_${i} already exists!
	fi
    done
}

setup_bridges_with_ip_command() {
    for i in `seq 8`
    do
	if [ ! -d /sys/class/net/Quwan_${i} ] ; then
	    ip link add name Quwan_${i} type bridge
	    echo Quwan_${i} added successfully!
	else
	    echo Quwan_${i} already exists!
	fi

	if [ "$(cat /sys/class/net/Quwan_${i}/operstate)" != "up" ]; then
	    ip link set dev Quwan_${i} up
	fi
    done
}

setup_bridges_with_brctl_command() {
    for i in `seq 8`
    do
	if [ ! -d /sys/class/net/Quwan_${i} ] ; then
	    brctl addbr Quwan_${i}
	    echo Quwan_${i} added successfully!
	else
	    echo Quwan_${i} already exists!
	fi

	if [ "$(cat /sys/class/net/Quwan_${i}/operstate)" != "up" ]; then
	    ifconfig Quwan_${i} up
	fi
    done
}

check_command_to_set_bridge() {
    if type nmcli >/dev/null 2>&1; then
	setup_bridges_with_nmcli_command
    elif type ip >/dev/null 2>&1; then
	setup_bridges_with_ip_command
    elif type brctl >/dev/null 2>&1 && type ifconfig >/dev/null 2>&1; then
	setup_bridges_with_brctl_command
    else
	echo Setup bridges failed. Make sure your network-manager, BRCTL or IP command is installed.
	exit
    fi
}

run_vm() {
    if type virsh >/dev/null 2>&1; then
	virsh define ${XML_NAME}
    else
	echo "Make sure your virsh command is installed."
    fi
}

# ===== main function ==== #

if test -n "${QCOW2_LOC}" ; then
    gen_xml
    modify_qcow2_path
    check_command_to_set_bridge
    run_vm
else
    echo Not Found the ${QCOW2_NAME} file.
fi

